import datetime
import os
import random
import time
from collections import Iterable

import pytz

from constants import db_params
from db_helper import MySqlDataStore, create_table_query

numbers = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

my_test_host, my_test_port = '192.168.0.11', 8000


def get_host():
    if 'IS_LAPTOP' not in os.environ:
        return 'https://payments.tennyapps.com/'
    else:
        return ''.join(['http://', my_test_host, ':', str(my_test_port), '/'])


def flatten(items, ignore_types=(str, bytes)):
    for x in items:
        if isinstance(x, Iterable) and not isinstance(x, ignore_types):
            for i in flatten(x):
                yield i
        else:
            yield x


def list_merger(given_list):
    new_list = []
    ignore_types = (str, bytes)
    for item in given_list:
        if isinstance(item, Iterable) and not isinstance(item, ignore_types):
            for i in list_merger(item):
                new_list.append(i)
        else:
            new_list.append(item)
    return new_list


def get_random_unique(count):
    mlist = numbers
    result = list()
    segments = 4
    loop = 0

    for n in range(count):
        code = ''

        for i in range(segments):
            indexes = random.sample(range(len(mlist)), random.sample([4, 5], 1)[0])
            for i in indexes:
                code += str(mlist[i])
            loop += 1
            if loop < segments:
                code += '-'
        result.append(code)
        loop = 0

    return list(set(result))


def generate_pay_id():
    mlist = numbers
    segments = 2
    loop = 0

    code = ''

    for i in range(segments):
        indexes = random.sample(range(len(mlist)), 3)
        for i in indexes:
            code += str(mlist[i])
        loop += 1
        if loop < segments:
            code += '-'

    return code


def byte_to_string(byte_dict):
    mdict = dict()

    for k, v in byte_dict.items():
        mdict[k.decode('utf8')] = v[0].decode('utf8')

    return mdict


def get_datastore():
    datastore = MySqlDataStore(host=db_params['host'], user=db_params['user'], passwd=db_params['passwd'],
                               db=db_params['db'])
    ''' create tables '''

    # users table
    table_name = 'users'
    table_column_dict = dict(
        id="int(11) PRIMARY KEY NOT NULL AUTO_INCREMENT",
        name="text",
        surname="text",
        phone_number="varchar(20) UNIQUE",
        access_token="varchar(100) UNIQUE",
        created="timestamp NUll",
        uuid="varchar(100)",
        password="varchar(100)"
    )
    datastore.create_table(create_table_query(table_name=table_name, column_dict=table_column_dict))

    # requests table
    table_name = 'requests'
    table_column_dict = dict(
        id=" int(11) PRIMARY KEY NOT NULL AUTO_INCREMENT",
        request_id="varchar(100) UNIQUE",
        phone_number="varchar(20)",
        access_token=" varchar(100)",
        package_name="text",
        sku="text",
        created="timestamp NULL",
        amount="double",
        period="int",
        approved="enum('Y','N') NOT NULL DEFAULT 'N'",
        pay_id='varchar(20)',
        consumed="enum('Y','N') NOT NULL DEFAULT 'N'",
        consume_date="text",
        expiry_date="text",
    )
    datastore.create_table(create_table_query(table_name=table_name, column_dict=table_column_dict))

    # sku table
    table_name = 'skues'
    table_column_dict = dict(
        id="int(11) PRIMARY KEY NOT NULL AUTO_INCREMENT",
        package_name="text",
        name="text",
        title="text",
        description="text",
        price="double"
    )

    datastore.create_table(create_table_query(table_name=table_name, column_dict=table_column_dict))

    # apps table
    table_name = 'apps'
    table_column_dict = dict(
        id="int(11) PRIMARY KEY NOT NULL AUTO_INCREMENT",
        package_name="text",
        version_code="int(11)",
        app_name="text",
        phone_number="varchar(20)"
    )

    datastore.create_table(create_table_query(table_name=table_name, column_dict=table_column_dict))

    return datastore


def generate_access_token():
    import secrets
    return secrets.token_urlsafe()


def get_current_time_in_millis():
    # return lambda: int(round(time.time() * 1000))
    return time.time() * 1000


def get_time_from_timestamp(millis):
    return datetime.datetime.fromtimestamp(millis / 1000, pytz.timezone("Africa/Harare")).strftime('%Y-%m-%d %H:%M:%S')


def get_now_timestamp():
    return datetime.datetime.fromtimestamp(time.time(), pytz.timezone("Africa/Harare")).strftime('%Y-%m-%d %H:%M:%S')


def generate_expiry_date(period):
    period_millis = period * 24 * 60 * 60 * 1000
    millis = get_current_time_in_millis() + period_millis
    return datetime.datetime.fromtimestamp(millis / 1000, pytz.timezone("Africa/Harare")).strftime('%Y-%m-%d %H:%M:%S')


def str_to_millis(s):
    return datetime.datetime.strptime(s, '%Y-%m-%d %H:%M:%S').timestamp() * 1000


if __name__ == '__main__':
    datastore = get_datastore()

    # generate recharge codes and insert in recharge_codes

    '''
    millis = get_current_time_in_millis()
    print(millis)
    ts = get_time_from_timestamp(millis)
    print(ts)
    expiry_date = datetime.datetime(2021, 5, 7, 10, 59, 00)
    print(expiry_date, type(expiry_date))'''

    start = time.time()
    millis = get_current_time_in_millis()
    str = get_time_from_timestamp(millis)
    print(str)
    new_millis = str_to_millis(str)
    print(get_time_from_timestamp(1621116068000))
    consume_date = get_now_timestamp()
    expiry_date = generate_expiry_date(1)
